﻿using UnityEngine;
using System.Collections;

public class Dude : MonoBehaviour {
	// Probability of making a visual cue instead of an audio one
	[Range(0.0f, 1.0f)] public float fVisualCueProbability = 0.5f;
	// Probability that a cue is just a bluff
	[Range(0.0f, 1.0f)] public float fBluffProbability = 0.3f;

	// Range values for the cue cooldown.
	// These determine how often the Dude will trigger cues
	public float fMinCueCooldown = 5.0f;
	public float fMaxCueCooldown = 9.0f;

	// Random slap time range
	public float fMinSlapDelay = 1.0f;
	public float fMaxSlapDelay = 3.0f;

	// Is one of the triggers active
	private bool bCueTriggering = false;
	private float fCueTimer = 1.0f;
	// Cooldown between cues
	private float fCueCooldown = 0.0f;

	// Is a slap coming as soon the delay is done
	private bool bSlapImminent = false;
	private float fSlapDelayTimer = 0.0f;
	private bool bIsSlapping = false;
	private bool bIsResetting = false;
	private float fSlapTimer = 0.0f;



	// Use this for initialization
	void Start () {

	}

	public bool SlapImminent() {return bSlapImminent;}

	public bool IsSlapping() {return bIsSlapping;}

	public bool IsCueing() {return bCueTriggering;}
	
	// Update is called once per frame
	void Update () {
		if (GetComponentInChildren<Animator> ().GetBool("EyeOpen") == true)
			GetComponentInChildren<Animator> ().SetBool ("EyeOpen", false);

		UpdateCue();
		UpdateSlapping();
	}

	void UpdateCue () {
		fCueTimer -= Time.deltaTime;

		// Check the timer that tells us the cue is still active
		if (fCueTimer > 0.0f)
			return;
		// We are no longer triggering
		bCueTriggering = false;

		fCueCooldown -= Time.deltaTime;

		// Wait for the cooldown before we trigger the slapping again
		if (fCueCooldown <= 0.0f)
		{
			print ("(*)");
			bCueTriggering = true;
			TriggerCue();
			fCueCooldown = Random.Range(fMinCueCooldown, fMaxCueCooldown);
		}

	}

	void TriggerCue(){
		fCueTimer = 1.5f;
		// Decide what kind of trigger we are going to cue
		float fRandomTrigger = Random.Range(0.0f, 1.0f);
		if (fRandomTrigger <= fVisualCueProbability)
			TriggerVisualCue();
		else
			TriggerAudioCue();

		// Generate a random number to decide if we are slapping or bluffing
		float fTriggerSlap = Random.Range(0.0f, 1.0f);

		if (fTriggerSlap >= fBluffProbability)
			bSlapImminent = true;
	}

	void TriggerVisualCue(){
		// Trigger some shit
		//print ("Trigger Visual");
		GetComponentInChildren<Animator> ().SetBool ("EyeOpen", true);
	}
	
	void TriggerAudioCue(){
		// Trigger some shit
		//print ("Trigger audio");
		GetComponentInChildren<Animator> ().SetBool ("EyeOpen", true);
	}

	void UpdateSlapping(){
		fSlapTimer -= Time.deltaTime;

		if (fSlapTimer <= 0.0f)
			bIsSlapping = false;

		// If we don't yet have imminent slapping or we are slapping return
		if (!bSlapImminent || bIsSlapping)
			return;

		// Count down until we trigger the slap
		fSlapDelayTimer -= Time.deltaTime;

		if (fSlapDelayTimer <= 0.0f)
		{
			bSlapImminent = false;
			Slap();
			fSlapDelayTimer = Random.Range(fMinSlapDelay, fMaxSlapDelay);
		}
	}

	void Slap(){
		bIsSlapping = true;
		fSlapTimer = 1.0f;
		bSlapImminent = false;
		print ("SLAAAAAAAAAAAAAAAAAAAAAAAAAAP!");

		SlapMozzies ();
		//animate
		HandController hand = GetComponentInChildren<HandController> ();
		hand.SlapIt ();
	}

	private void SlapMozzies()
	{
		Mozzie[] aMozzies;
		aMozzies = FindObjectsOfType(typeof(Mozzie)) as Mozzie[];

		for(uint i = 0; i < aMozzies.Length; i++)
		{
			aMozzies[i].Slap();
		}
	}

}
