﻿using UnityEngine;
using System.Collections;

public class Mozzie : MonoBehaviour {
	/* The way it works:
	 * Sucks blood at linear rate...
	 * Elvis wakes up:
	 * Suck blood faster (exponential rate)
	 * Elvis may slap:
	 * Loose all blood from round + be dazed for a few seconds
	 * Mozzie Jumps:
	 * Banks blood sucked in round to total points + safe from slap
	 * 
	 */


	private float fTotalPoints = 0, fLinearRoundPoints = 0, fExponentialRoundPoints = 0, 
				  fRoundPoints = 0, fExponentialTime = 0, fIncrement = 0;
	protected bool bIsLanded = true,  bIsAlive = true, bUseLinearRate = false;
	public bool DebugEnabled = false;

	public float fLinearRate = 1f, fExponentialRate = 1f, LaunchForce = 500f;

	public float DazeTime = 1.0f;
	private float _dazeRemaining = 0.0f;

	private Animator _wingsAnimator;
	private Animator _bodyAnimator;

	//public BloodProgress LinkedButton;

	public int GetRoundPoints { get { return (int)fRoundPoints; } }
	public int GetTotalPoints { get { return (int)fTotalPoints; } }
	public float GetDazeRemaining { get { return _dazeRemaining; } }

	// Use this for initialization
	protected virtual void Start () {
		_wingsAnimator = GetGameObjectByName ("Wings").GetComponentInChildren<Animator>();
		_bodyAnimator = GetGameObjectByName ("Body").GetComponentInChildren<Animator>();
	}
	
	// Update is called once per frame
	void Update () {
		if(bIsAlive)
		{
			SuckBlood();
			OnUpdate();
			//CheckLaunchKey(); //Moved launch to buttons
		}
	}

	protected virtual void OnUpdate()
	{
	}

	void SuckBlood()
	{
		if(!bIsLanded) return;

		if (_dazeRemaining > 0.0f) 
		{
			_dazeRemaining -= Time.deltaTime;

			if (_dazeRemaining <= 0.1f)
			{
				GetGameObjectByName ("EyeL").GetComponent<Bobber>().Pause = false;
				GetGameObjectByName ("EyeR").GetComponent<Bobber>().Pause = false;
				GetGameObjectByName ("PupilL").GetComponent<Bobber>().Circles = false;
				GetGameObjectByName ("PupilR").GetComponent<Bobber>().Circles = false;
			}

			return;
		}

		//probably not great to keep setting it. whatever.
		_bodyAnimator.SetBool ("IsSucking", true);

		if(bUseLinearRate)
		{
			fIncrement = Time.deltaTime * fLinearRate;
		}
		else
		{
			fIncrement = Time.deltaTime * fExponentialRate/(1 + Mathf.Exp(-Time.timeSinceLevelLoad + fExponentialTime));
		}

		fRoundPoints += fIncrement;
	}

	public void Launch()
	{
		if ((bIsLanded) && (_dazeRemaining <= 0.0f))
		{
			ChangeLandedState(false);
			//rigidbody.velocity = new Vector3(0,fLaunchVelocity,0);
			rigidbody.AddForce(0,LaunchForce,0);

			_bodyAnimator.SetBool ("IsSucking", false);

			//allocate points
			fTotalPoints += fRoundPoints;
			fRoundPoints = 0.0f;
		}
	}

	void ChangeLandedState(bool bNewState)
	{
		if(bNewState && !bIsLanded)
		{
			fExponentialTime = Time.timeSinceLevelLoad;
		}
		bIsLanded = bNewState;

		_wingsAnimator.SetBool ("IsFlying", !bIsLanded);
	}

	GameObject GetGameObjectByName(string name)
	{
		Transform[] transforms = GetComponentsInChildren<Transform>();
		foreach (Transform t in transforms)
		{
			if (t.name == name)
			{
				return t.gameObject;
			}
		}
		
		return null;
	}

	void OnCollisionEnter(Collision c)
	{
		if(c.gameObject.GetComponents<LandingPad>() != null)
		{
			ChangeLandedState(true);
		}
	}

	public void Slap()
	{
		if(bIsLanded)
		{
			_dazeRemaining = DazeTime;

			_bodyAnimator.SetBool ("IsSucking", false);
			//TODO: set a 'very dazed' animation

			GetGameObjectByName ("EyeL").GetComponent<Bobber>().Pause = true;
			GetGameObjectByName ("EyeR").GetComponent<Bobber>().Pause = true;
			GetGameObjectByName ("PupilL").GetComponent<Bobber>().Circles = true;
			GetGameObjectByName ("PupilR").GetComponent<Bobber>().Circles = true;

			//lose round points
			fRoundPoints = 0.0f;
		}
	}

	public void SetExponentialTime(bool bIsExpTime)
	{
		bUseLinearRate = !bIsExpTime;
		fExponentialTime = Time.timeSinceLevelLoad;
	}

	void OnGUI()
	{
		if(!DebugEnabled) return;
		if(GUI.Button(new Rect(0,0,100,100),"DEBUG"))
		{
			SetExponentialTime(bUseLinearRate);
		}
	}
}
