﻿using UnityEngine;
using System.Collections;

public class NPC_Mozzie : Mozzie {
	// Default launch times
	[Range(0.0f, 10.0f)] public float fMinLaunchTime = 15.0f;
	[Range(10.0f, 100.0f)] public float fMaxLaunchTime = 20.0f;

	// Launch times when we decide that the Dude is going to slap
	[Range(0.0f, 10.0f)] public float fMinLaunchTimePostTrigger = 0.2f;
	[Range(0.0f, 10.0f)] public float fMaxLaunchTimePostTrigger = 2.0f;

	// Probability of launching when we get a cue from the Dude
	[Range(0.0f, 1.0f)] public float fLaunchProbabilityWhenTrigger = 0.6f;

	private Dude theDude;
	private float fLaunchTime = 0.0f;
	private float fLaunchTimePostTrigger = 0.0f;

	private float fTimeToLaunch = 0.0f;

	private bool bTriggerDecisionMade = false;

	// Use this for initialization
	protected override void Start () {
		theDude = FindObjectOfType(typeof(Dude)) as Dude;
		fTimeToLaunch = Random.Range(fMinLaunchTime, fMaxLaunchTime);
	}

	protected override void OnUpdate()
	{
		// Only check for launch when we are on the Dude, drinking blood
		if (!bIsLanded)
			return;

		fTimeToLaunch -= Time.deltaTime;

		fLaunchTimePostTrigger = Random.Range(fMinLaunchTimePostTrigger, fMaxLaunchTimePostTrigger);

		if (theDude.SlapImminent() && !bTriggerDecisionMade)
		{
			bTriggerDecisionMade = true;
			print ("Slap Imminent");
			fTimeToLaunch = (fTimeToLaunch > fLaunchTimePostTrigger)? fLaunchTimePostTrigger : fTimeToLaunch;
		}

		if (theDude.IsCueing() && !bTriggerDecisionMade)
		{
			print ("Cue slap");
			bTriggerDecisionMade = true;
			float fProb = Random.Range(0.0f, 1.0f);
			fTimeToLaunch = ((fLaunchProbabilityWhenTrigger >= fProb)&& (fTimeToLaunch > fLaunchTimePostTrigger)) ? fLaunchTimePostTrigger : fTimeToLaunch;
		}

		// Wait before we launch
		if (fTimeToLaunch <= 0.0f)
		{
			bTriggerDecisionMade = false;
			fTimeToLaunch = Random.Range(fMinLaunchTime, fMaxLaunchTime);
			Launch();
		}
	}

}
